/*******************************************************************************
* Copyright 2019-2021 Intel Corporation
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*     http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*******************************************************************************/

#include <cassert>
#include <string>
#include <unordered_map>
#include <vector>

#include "gpu/ocl/kernel_utils.hpp"

namespace dnnl {
namespace impl {
namespace gpu {
namespace ocl {


extern const char *combined_reduction_kernel;
extern const char *custom_reorder_kernel;
extern const char *gemm_with_post_ops_kernel;
extern const char *gen9_gemm_beta_kernel;
extern const char *gen9_gemm_compute_kernel;
extern const char *gen9_gemm_copy_kernel;
extern const char *gen9_gemm_nocopy_f16_kernel;
extern const char *gen9_gemm_nocopy_f32_kernel;
extern const char *gen9_gemm_nocopy_scale_x8x8s32_kernel;
extern const char *gen9_gemm_nocopy_superkernel_f32_kernel;
extern const char *gen9_gemm_nocopy_x8x8s32_kernel;
extern const char *ref_gemm_kernel;
extern const char *xe_hp_systolic_gemm_copy_kernel;
extern const char *xe_hpc_systolic_gemm_copy_kernel;
extern const char *xe_lp_gemm_nocopy_scale_x8x8s32_kernel;
extern const char *xe_lp_gemm_nocopy_x8x8s32_kernel;
extern const char *gemm_post_ops_inner_product_kernel;
extern const char *gen9_binary_kernel;
extern const char *gen9_bnorm_kernel;
extern const char *gen9_concat_kernel;
extern const char *gen9_conv_bwd_data_kernel;
extern const char *gen9_conv_bwd_weights_kernel;
extern const char *gen9_conv_dw_bwd_data_kernel;
extern const char *gen9_conv_dw_fwd_data_kernel;
extern const char *gen9_conv_fwd_data_kernel;
extern const char *gen9_conv_nhwc_bwd_data_kernel;
extern const char *gen9_conv_nhwc_bwd_weights_f32_kernel;
extern const char *gen9_conv_nhwc_fwd_data_kernel;
extern const char *gen9_eltwise_kernel;
extern const char *gen9_global_pooling_kernel;
extern const char *gen9_pooling_kernel;
extern const char *gen9_reduction_kernel;
extern const char *gen9_softmax_kernel;
extern const char *gen9_sum_kernel;
extern const char *gen9_wino_conv_fwd_data_2x3_kernel;
extern const char *gen9_wino_conv_fwd_data_fused_kernel;
extern const char *generic_reorder_kernel;
extern const char *many_inputs_sum_kernel;
extern const char *ref_binary_kernel;
extern const char *ref_bnorm_kernel;
extern const char *ref_convolution_kernel;
extern const char *ref_deconv_backward_bias_kernel;
extern const char *ref_eltwise_kernel;
extern const char *ref_inner_product_kernel;
extern const char *ref_layer_normalization_kernel;
extern const char *ref_lrn_kernel;
extern const char *ref_matmul_kernel;
extern const char *ref_pooling_kernel;
extern const char *ref_prelu_kernel;
extern const char *ref_reduction_kernel;
extern const char *ref_reorder_kernel;
extern const char *ref_resampling_kernel;
extern const char *ref_shuffle_kernel;
extern const char *ref_softmax_kernel;
extern const char *ref_zero_pad_kernel;
extern const char *ref_rnn_kernel;
extern const char *rnn_reorder_kernel;
extern const char *simple_concat_kernel;
extern const char *simple_sum_kernel;
extern const char *vectorized_resampling_kernel;
extern const char *xe_lp_1x1_conv_fwd_data_x8s8x_kernel;
extern const char *xe_lp_conv_bwd_data_mb_block_x8s8x8_kernel;
extern const char *xe_lp_conv_bwd_data_x8s8x8_kernel;
extern const char *xe_lp_conv_dw_fwd_data_mb_block_x8s8x_kernel;
extern const char *xe_lp_conv_dw_fwd_data_ow_block_x8s8x_kernel;
extern const char *xe_lp_conv_fwd_data_first_x8s8x_kernel;
extern const char *xe_lp_conv_fwd_data_mb_block_x8s8x_kernel;
extern const char *xe_lp_conv_fwd_data_ow_block_x8s8x_kernel;
extern const char *xe_lp_conv_nhwc_fwd_dw_mb_block_x8s8x_kernel;
extern const char *xe_lp_conv_nhwc_fwd_dw_ow_block_x8s8x_kernel;
extern const char *xe_lp_conv_nhwc_fwd_first_x8s8x_kernel;
extern const char *xe_lp_conv_nhwc_fwd_x8s8x_kernel;
extern const char *xe_lp_nhwc_1x1_conv_fwd_x8s8x_kernel;
extern const char *xe_lp_x8s8x_compensation_kernel;

extern const char *binary_types_header;
extern const char *ocl_gemm_attrs_header;
extern const char *metrics_discovery_api_header;
extern const char *ocl_eltwise_header;
extern const char *ocl_math_utils_header;
extern const char *ocl_post_ops_header;
extern const char *ocl_types_header;
extern const char *ocl_zero_points_header;
extern const char *offsets_header;
extern const char *reorder_common_header;
extern const char *rnn_types_header;
extern const char *zero_pad_struct_header;

const char *get_kernel_source(const char *name) {
    static const std::unordered_map<std::string, const char *> kernel_list = {
        
        { "combined_reduce", combined_reduction_kernel },
        { "custom_reorder", custom_reorder_kernel },
        { "gemm_post_ops", gemm_with_post_ops_kernel },
        { "gen9_gemm_beta", gen9_gemm_beta_kernel },
        { "gen9_gemm_compute", gen9_gemm_compute_kernel },
        { "gen9_gemm_copy", gen9_gemm_copy_kernel },
        { "gen9_gemm_nocopy_f16", gen9_gemm_nocopy_f16_kernel },
        { "gen9_gemm_nocopy_f16", gen9_gemm_nocopy_f16_kernel },
        { "gen9_gemm_nocopy_f16", gen9_gemm_nocopy_f16_kernel },
        { "gen9_gemm_nocopy_f16", gen9_gemm_nocopy_f16_kernel },
        { "gen9_gemm_nocopy_f32", gen9_gemm_nocopy_f32_kernel },
        { "gen9_gemm_nocopy_f32", gen9_gemm_nocopy_f32_kernel },
        { "gen9_gemm_nocopy_f32", gen9_gemm_nocopy_f32_kernel },
        { "gen9_gemm_nocopy_f32", gen9_gemm_nocopy_f32_kernel },
        { "gen9_gemm_scale_x8x8s32", gen9_gemm_nocopy_scale_x8x8s32_kernel },
        { "gen9_gemm_nocopy_superkernel_f32", gen9_gemm_nocopy_superkernel_f32_kernel },
        { "gen9_gemm_nocopy_superkernel_f32", gen9_gemm_nocopy_superkernel_f32_kernel },
        { "gen9_gemm_compute_x8x8s32", gen9_gemm_nocopy_x8x8s32_kernel },
        { "ref_gemm", ref_gemm_kernel },
        { "xe_hp_systolic_gemm_copy", xe_hp_systolic_gemm_copy_kernel },
        { "xe_hp_systolic_gemm_copy", xe_hp_systolic_gemm_copy_kernel },
        { "xe_hp_systolic_gemm_copy", xe_hp_systolic_gemm_copy_kernel },
        { "xe_hp_systolic_gemm_copy", xe_hp_systolic_gemm_copy_kernel },
        { "xe_hp_systolic_gemm_copy", xe_hp_systolic_gemm_copy_kernel },
        { "xe_hp_systolic_gemm_copy", xe_hp_systolic_gemm_copy_kernel },
        { "xe_hpc_systolic_gemm_copy", xe_hpc_systolic_gemm_copy_kernel },
        { "xe_hpc_systolic_gemm_copy", xe_hpc_systolic_gemm_copy_kernel },
        { "xe_hpc_systolic_gemm_copy", xe_hpc_systolic_gemm_copy_kernel },
        { "xe_hpc_systolic_gemm_copy", xe_hpc_systolic_gemm_copy_kernel },
        { "xe_hpc_systolic_gemm_copy", xe_hpc_systolic_gemm_copy_kernel },
        { "xe_hpc_systolic_gemm_copy", xe_hpc_systolic_gemm_copy_kernel },
        { "xe_lp_gemm_scale_x8x8s32", xe_lp_gemm_nocopy_scale_x8x8s32_kernel },
        { "xe_lp_gemm_compute_x8x8s32", xe_lp_gemm_nocopy_x8x8s32_kernel },
        { "xe_lp_gemm_compute_x8x8s32", xe_lp_gemm_nocopy_x8x8s32_kernel },
        { "xe_lp_gemm_compute_x8x8s32", xe_lp_gemm_nocopy_x8x8s32_kernel },
        { "xe_lp_gemm_compute_x8x8s32", xe_lp_gemm_nocopy_x8x8s32_kernel },
        { "gemm_post_ops_inner_product", gemm_post_ops_inner_product_kernel },
        { "gen9_binary", gen9_binary_kernel },
        { "gen9_binary", gen9_binary_kernel },
        { "gen9_binary", gen9_binary_kernel },
        { "gen9_binary", gen9_binary_kernel },
        { "gen9_fused_reduce_init", gen9_bnorm_kernel },
        { "gen9_calc_mean_var", gen9_bnorm_kernel },
        { "gen9_calc_mean_var", gen9_bnorm_kernel },
        { "gen9_reduce_mean_var", gen9_bnorm_kernel },
        { "gen9_fused_reduce_final", gen9_bnorm_kernel },
        { "gen9_calc_mean", gen9_bnorm_kernel },
        { "gen9_calc_mean", gen9_bnorm_kernel },
        { "gen9_reduce_mean", gen9_bnorm_kernel },
        { "gen9_calc_variance", gen9_bnorm_kernel },
        { "gen9_calc_variance", gen9_bnorm_kernel },
        { "gen9_reduce_variance", gen9_bnorm_kernel },
        { "gen9_bnorm_fwd", gen9_bnorm_kernel },
        { "gen9_bnorm_fwd", gen9_bnorm_kernel },
        { "gen9_calculate_stats", gen9_bnorm_kernel },
        { "gen9_calculate_stats", gen9_bnorm_kernel },
        { "gen9_reduce_stats", gen9_bnorm_kernel },
        { "gen9_fused_reduce_final", gen9_bnorm_kernel },
        { "gen9_bnorm_bwd", gen9_bnorm_kernel },
        { "gen9_bnorm_bwd", gen9_bnorm_kernel },
        { "gen9_concat", gen9_concat_kernel },
        { "gen9_conv_bwd_data", gen9_conv_bwd_data_kernel },
        { "gen9_conv_bwd_weights", gen9_conv_bwd_weights_kernel },
        { "gen9_conv_dw_bwd_data", gen9_conv_dw_bwd_data_kernel },
        { "gen9_conv_dw_fwd", gen9_conv_dw_fwd_data_kernel },
        { "gen9_conv_fwd", gen9_conv_fwd_data_kernel },
        { "gen9_conv_nhwc_bwd_data", gen9_conv_nhwc_bwd_data_kernel },
        { "gen9_conv_nhwc_bwd_weights", gen9_conv_nhwc_bwd_weights_f32_kernel },
        { "gen9_conv_nhwc_fwd", gen9_conv_nhwc_fwd_data_kernel },
        { "gen9_eltwise_fwd", gen9_eltwise_kernel },
        { "gen9_eltwise_bwd", gen9_eltwise_kernel },
        { "gen9_global_pooling_fwd", gen9_global_pooling_kernel },
        { "gen9_global_pooling_bwd", gen9_global_pooling_kernel },
        { "gen9_pooling_fwd", gen9_pooling_kernel },
        { "gen9_pooling_bwd", gen9_pooling_kernel },
        { "gen9_initial_reduce", gen9_reduction_kernel },
        { "gen9_final_reduce", gen9_reduction_kernel },
        { "gen9_softmax_fwd", gen9_softmax_kernel },
        { "gen9_softmax_bwd", gen9_softmax_kernel },
        { "gen9_sum", gen9_sum_kernel },
        { "gen9_wino_wei_transform_2x3", gen9_wino_conv_fwd_data_2x3_kernel },
        { "gen9_wino_src_transform_2x3", gen9_wino_conv_fwd_data_2x3_kernel },
        { "gen9_wino_dst_transform_2x3", gen9_wino_conv_fwd_data_2x3_kernel },
        { "gen9_wino_conv_fwd_2x3", gen9_wino_conv_fwd_data_2x3_kernel },
        { "gen9_wino_wei_transform", gen9_wino_conv_fwd_data_fused_kernel },
        { "gen9_wino_conv_fwd", gen9_wino_conv_fwd_data_fused_kernel },
        { "generic_reorder", generic_reorder_kernel },
        { "many_inputs_sum", many_inputs_sum_kernel },
        { "many_inputs_sum_batched", many_inputs_sum_kernel },
        { "ref_binary", ref_binary_kernel },
        { "ref_binary", ref_binary_kernel },
        { "calculate_mean", ref_bnorm_kernel },
        { "calculate_variance", ref_bnorm_kernel },
        { "calculate_mean_variance", ref_bnorm_kernel },
        { "calculate_mean", ref_bnorm_kernel },
        { "calculate_variance", ref_bnorm_kernel },
        { "reduce_mean", ref_bnorm_kernel },
        { "reduce_variance", ref_bnorm_kernel },
        { "ref_bnorm_fwd", ref_bnorm_kernel },
        { "calculate_stats", ref_bnorm_kernel },
        { "reduce_stats", ref_bnorm_kernel },
        { "calculate_stats", ref_bnorm_kernel },
        { "reduce_stats", ref_bnorm_kernel },
        { "ref_bnorm_bwd", ref_bnorm_kernel },
        { "ref_convolution_fwd", ref_convolution_kernel },
        { "ref_convolution_bwd_data", ref_convolution_kernel },
        { "ref_convolution_bwd_weights", ref_convolution_kernel },
        { "ref_deconv_backward_bias", ref_deconv_backward_bias_kernel },
        { "ref_eltwise_fwd", ref_eltwise_kernel },
        { "ref_eltwise_bwd", ref_eltwise_kernel },
        { "ref_inner_product_fwd", ref_inner_product_kernel },
        { "ref_inner_product_bwd_data", ref_inner_product_kernel },
        { "ref_inner_product_bwd_weights", ref_inner_product_kernel },
        { "ref_lnorm_fwd", ref_layer_normalization_kernel },
        { "ref_lnorm_fwd", ref_layer_normalization_kernel },
        { "ref_lnorm_bwd_scaleshift", ref_layer_normalization_kernel },
        { "ref_lnorm_bwd_scaleshift_final", ref_layer_normalization_kernel },
        { "ref_lnorm_bwd_scaleshift", ref_layer_normalization_kernel },
        { "ref_lnorm_bwd", ref_layer_normalization_kernel },
        { "ref_lnorm_bwd", ref_layer_normalization_kernel },
        { "ref_lrn_fwd", ref_lrn_kernel },
        { "ref_lrn_bwd", ref_lrn_kernel },
        { "ref_matmul", ref_matmul_kernel },
        { "ref_pooling_fwd", ref_pooling_kernel },
        { "ref_pooling_bwd", ref_pooling_kernel },
        { "ref_prelu_fwd", ref_prelu_kernel },
        { "ref_prelu_bwd", ref_prelu_kernel },
        { "ref_reduce", ref_reduction_kernel },
        { "ref_reorder", ref_reorder_kernel },
        { "ref_resampling_fwd", ref_resampling_kernel },
        { "ref_resampling_bwd", ref_resampling_kernel },
        { "ref_shuffle", ref_shuffle_kernel },
        { "ref_softmax_fwd_generic", ref_softmax_kernel },
        { "ref_softmax_bwd_generic", ref_softmax_kernel },
        { "ref_zero_pad", ref_zero_pad_kernel },
        { "ref_zero_pad_subg_16", ref_zero_pad_kernel },
        { "ref_zero_pad_subg_16_mask_and_clear_dt_1b", ref_zero_pad_kernel },
        { "ref_rnn_copy_init_layer", ref_rnn_kernel },
        { "ref_rnn_copy_init_iter", ref_rnn_kernel },
        { "ref_rnn_copy_res_layer", ref_rnn_kernel },
        { "ref_rnn_copy_res_iter", ref_rnn_kernel },
        { "ref_rnn_ws_set", ref_rnn_kernel },
        { "ref_rnn_ws_print", ref_rnn_kernel },
        { "ref_rnn_bias_prepare", ref_rnn_kernel },
        { "ref_rnn_elemwise_fwd", ref_rnn_kernel },
        { "ref_rnn_elemwise_fwd", ref_rnn_kernel },
        { "ref_rnn_elemwise_bwd", ref_rnn_kernel },
        { "ref_rnn_gates_reduction", ref_rnn_kernel },
        { "wei_reorder", rnn_reorder_kernel },
        { "simple_concat", simple_concat_kernel },
        { "simple_sum", simple_sum_kernel },
        { "vectorized_resampling_bwd", vectorized_resampling_kernel },
        { "xe_lp_1x1_conv_fwd_x8s8x", xe_lp_1x1_conv_fwd_data_x8s8x_kernel },
        { "conv_bwd_data_mb_block_x8s8x8", xe_lp_conv_bwd_data_mb_block_x8s8x8_kernel },
        { "conv_bwd_data_x8s8x8", xe_lp_conv_bwd_data_x8s8x8_kernel },
        { "conv_dw_fwd_mb_block_x8s8x", xe_lp_conv_dw_fwd_data_mb_block_x8s8x_kernel },
        { "conv_dw_fwd_ow_block_x8s8x", xe_lp_conv_dw_fwd_data_ow_block_x8s8x_kernel },
        { "conv_fwd_first_x8s8x", xe_lp_conv_fwd_data_first_x8s8x_kernel },
        { "conv_fwd_mb_block_x8s8x", xe_lp_conv_fwd_data_mb_block_x8s8x_kernel },
        { "conv_fwd_ow_block_x8s8x", xe_lp_conv_fwd_data_ow_block_x8s8x_kernel },
        { "conv_nhwc_fwd_dw_mb_block_x8s8x", xe_lp_conv_nhwc_fwd_dw_mb_block_x8s8x_kernel },
        { "conv_nhwc_fwd_dw_ow_block_x8s8x", xe_lp_conv_nhwc_fwd_dw_ow_block_x8s8x_kernel },
        { "conv_nhwc_fwd_first_x8s8x", xe_lp_conv_nhwc_fwd_first_x8s8x_kernel },
        { "conv_nhwc_fwd_x8s8x", xe_lp_conv_nhwc_fwd_x8s8x_kernel },
        { "xe_lp_nhwc_1x1_conv_fwd_x8s8x", xe_lp_nhwc_1x1_conv_fwd_x8s8x_kernel },
        { "xe_lp_x8s8x_compensation", xe_lp_x8s8x_compensation_kernel },
        { "xe_lp_x8s8x_compensation", xe_lp_x8s8x_compensation_kernel },
    };

    if (!name) return nullptr;

    assert(kernel_list.count(name) == 1);
    return kernel_list.at(name);
}

const char *get_kernel_header(const std::string &name) {
    static const std::unordered_map<std::string, const char *> kernel_header_list ={
        
        {"gpu/ocl/binary_types.h", binary_types_header},
        {"gpu/ocl/gemm/ocl_gemm_attrs.h", ocl_gemm_attrs_header},
        {"gpu/ocl/mdapi/metrics_discovery_api.h", metrics_discovery_api_header},
        {"gpu/ocl/ocl_eltwise.h", ocl_eltwise_header},
        {"gpu/ocl/ocl_math_utils.h", ocl_math_utils_header},
        {"gpu/ocl/ocl_post_ops.h", ocl_post_ops_header},
        {"gpu/ocl/ocl_types.h", ocl_types_header},
        {"gpu/ocl/ocl_zero_points.h", ocl_zero_points_header},
        {"gpu/ocl/offsets.h", offsets_header},
        {"gpu/ocl/reorder_common.h", reorder_common_header},
        {"gpu/ocl/rnn/rnn_types.h", rnn_types_header},
        {"gpu/zero_pad_struct.h", zero_pad_struct_header},
    };

    assert(kernel_header_list.count(name) == 1);
    return kernel_header_list.at(name);
}

} // namespace ocl
} // namespace gpu
} // namespace impl
} // namespace dnnl
